﻿using System.Text;
using CoreMvcApplication.Models.AlipayViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class AlipayController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public AlipayController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<AlipayController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.Alipay.SuccessUrl,
            ErrorUrl = _settings.Alipay.ErrorUrl,
            CommodityName = "新製品",
            CommodityDescription = "新製品詳細"
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (_settings.Mdk.Dummy && model.Currency == "CNY" && model.Amount < 100)
            ModelState.AddModelError("Amount", "金額には100以上の値を指定してください");

        switch (model.PayType)
        {
            case "0":
                ModelState.Remove("CommodityId");
                ModelState.Remove("IdentityCode");
                ModelState.Remove("ResponseType");
                break;
            case "1":
                ModelState.Remove("SuccessUrl");
                ModelState.Remove("ErrorUrl");
                ModelState.Remove("CommodityDescription");
                ModelState.Remove("CommodityId");
                break;
            case "2":
                ModelState.Remove("SuccessUrl");
                ModelState.Remove("ErrorUrl");
                ModelState.Remove("IdentityCode");
                ModelState.Remove("ResponseType");
                break;
        }


        if (ModelState.IsValid)
        {
            var requestDto = new AlipayAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Currency = model.Currency,
                CommodityName = model.CommodityName,
                PayType = model.PayType
            };

            switch (model.PayType)
            {
                case "0":
                    requestDto.SuccessUrl = model.SuccessUrl;
                    requestDto.ErrorUrl = model.ErrorUrl;
                    requestDto.CommodityDescription = model.CommodityDescription;
                    break;
                case "1":
                    requestDto.IdentityCode = model.IdentityCode;
                    requestDto.ResponseType = model.ResponseType;
                    break;
                case "2":
                    requestDto.CommodityDescription = model.CommodityDescription;
                    requestDto.CommodityId = model.CommodityId;
                    break;
            }

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AlipayAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeExec", new {orderId = model.OrderId, payType = model.PayType});
        }


        return View(model);
    }


    [HttpGet]
    public IActionResult AuthorizeExec(string orderId, string payType)
    {
        var model = new ExecViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<AlipayAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.EntryForm = responseData.EntryForm;
            model.CustTxn = responseData.CustTxn;
            model.CenterTradeId = responseData.CenterTradeId;
            model.PayTimeJp = responseData.PayTimeJp;
            model.PayTimeCn = responseData.PayTimeCn;
            model.BuyerChargedAmountCny = responseData.BuyerChargedAmountCny;
            model.QrCode = responseData.QrCode;
            model.QrCodeImgUrl = responseData.QrCodeImgUrl;
            model.QrCodeSmallImgUrl = responseData.QrCodeSmallImgUrl;
            model.QrCodeLargeImgUrl = responseData.QrCodeLargeImgUrl;
            model.PayType = payType;
        }

        return View(model);
    }


    [HttpPost]
    public IActionResult AuthorizeResult(
        [FromForm(Name = "serviceType")] string serviceType,
        [FromForm(Name = "authInfo")] string authInfo,
        [FromForm(Name = "mstatus")] string mstatus,
        [FromForm(Name = "vResultCode")] string vResultCode,
        [FromForm(Name = "merrMsg")] string merrMsg,
        [FromForm(Name = "custTxn")] string custTxn,
        [FromForm(Name = "orderId")] string orderId,
        [FromForm(Name = "centerTradeId")] string centerTradeId,
        [FromForm(Name = "settleAmount")] string settleAmount,
        [FromForm(Name = "settleCurrency")] string settleCurrency
    )
    {
        var model = new ResultViewModel();

        if (!string.IsNullOrEmpty(authInfo))
        {
            var keys = authInfo.Split('-');
            if (keys.Length == 3)
            {
                var merchantCcid = Encoding.ASCII.GetString(Util.FromBase64(keys[0]));
                var now = Encoding.ASCII.GetString(Util.FromBase64(keys[1]));
                var receivedHash = Encoding.ASCII.GetString(Util.FromBase64(keys[2]));

                var hashedData =
                    HashUtils.GetInstance.GetDigest(new[] {merchantCcid, now, _settings.Mdk.MerchantSecret});

                if (hashedData == receivedHash)
                {
                    model.OrderId = orderId;
                    model.Mstatus = mstatus;
                    model.VResultCode = vResultCode;
                    model.MerrMsg = merrMsg;
                    model.CustTxn = custTxn;
                    model.SettleAmount = settleAmount;
                    model.SettleCurrency = settleCurrency;
                }
                else
                {
                    ModelState.AddModelError("AuthInfoError", "認証に失敗しました。改竄された恐れがあります。");
                }
            }
            else
            {
                ModelState.AddModelError("AuthInfoFormatError", "認証情報パラメータ(authInfo)のフォーマットが間違っています。");
            }
        }
        else
        {
            ModelState.AddModelError("AuthInfoMissing", "認証情報パラメータ(authInfo)がありません。");
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Confirm()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Confirm(ConfirmViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AlipayConfirmRequestDto
            {
                OrderId = model.OrderId,
                ResponseType = model.ResponseType
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AlipayConfirmResponseDto>(tran, requestDto);

            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ConfirmResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ConfirmResult(string orderId)
    {
        var model = new ExecViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<AlipayConfirmResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CustTxn = responseData.CustTxn;
            model.CenterTradeId = responseData.CenterTradeId;
            model.PayTimeJp = responseData.PayTimeJp;
            model.PayTimeCn = responseData.PayTimeCn;
            model.BuyerChargedAmountCny = responseData.BuyerChargedAmountCny;
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AlipayRefundRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Reason = model.Reason
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AlipayRefundResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<AlipayRefundResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CenterTradeId = responseData.CenterTradeId;
            model.Balance = responseData.Balance;
        }

        return View(model);
    }
}