﻿using System.Security.Cryptography;
using System.Text;
using System.Text.Encodings.Web;
using System.Web;
using CoreMvcApplication.Models.AmazonPayViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class AmazonPayController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public AmazonPayController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<AmazonPayController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region Authorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.AmazonPay.SuccessUrl,
            CancelUrl = _settings.AmazonPay.CancelUrl,
            ErrorUrl = _settings.AmazonPay.ErrorUrl,
            PayConfirmScreenUrl = _settings.AmazonPay.PayConfirmScreenUrl
        };
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AmazonpayAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WithCapture = model.WithCapture,
                SuppressShippingAddressView = model.SuppressShippingAddressView,
                NoteToBuyer = model.NoteToBuyer,
                SuccessUrl = model.SuccessUrl ?? _settings.AmazonPay.SuccessUrl,
                CancelUrl = model.CancelUrl ?? _settings.AmazonPay.CancelUrl,
                ErrorUrl = model.ErrorUrl ?? _settings.AmazonPay.ErrorUrl,
                PayConfirmScreenType = model.PayConfirmScreenType,
                PayConfirmScreenUrl = model.PayConfirmScreenUrl ?? _settings.AmazonPay.PayConfirmScreenUrl,
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayAuthorizeResponseDto>(tran, requestDto);

            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            TempData[model.OrderId + "_AuthorizeRequest"] = JsonConvert.SerializeObject(requestDto);
            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayAuthorizeResponseDto>(tempData) : null;

            if (!string.IsNullOrWhiteSpace(responseData?.ResponseContents))
                return Content(responseData.ResponseContents, "text/html", Encoding.UTF8);

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }

    [HttpPost]
    public IActionResult Result([FromForm] IFormCollection posted)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(posted, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = posted["orderId"],
            Mstatus = posted["mstatus"],
            VResultCode = posted["vResultCode"],
            CenterOrderId = posted["centerOrderId"],
            CenterTransactionId = posted["centerTransactionId"],
            CenterResponseDatetime = posted["centerResponseDatetime"],
            ShippingName = posted["shippingName"],
            ShippingPhone = posted["shippingPhone"],
            ShippingAddress1 = posted["shippingAddress1"],
            ShippingAddress2 = posted["shippingAddress2"],
            ShippingAddress3 = posted["shippingAddress3"],
            ShippingPrefecture = posted["shippingPrefecture"],
            ShippingPostalCode = posted["shippingPostalCode"],
        };

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeConfirm([FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "orderHash")] string orderHash)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var hashKey = merchantCcId + orderId + merchantPw;
            var sha1 = SHA1.Create();
            var byteHashValue = sha1.ComputeHash(Encoding.UTF8.GetBytes(hashKey));
            var hashValue = BitConverter.ToString(byteHashValue).ToLower().Replace("-", "");
            // パラメータ改ざんチェック
            if (orderHash != hashValue)
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var getAddressRequestDto = new AmazonpayGetAddressRequestDto
        {
            OrderId = orderId,
            AddInfoRespFlag = "1"
        };
        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
        var getAddressResponseDto =
            _mdkTransactionService.ExecuteMdkTransaction<AmazonpayGetAddressResponseDto>(tran, getAddressRequestDto);

        if (getAddressResponseDto.Mstatus == "success")
        {
            TempData[orderId + "_GetAddressResponse"] = JsonConvert.SerializeObject(getAddressResponseDto);
            
            var tempData = TempData.Peek(orderId + "_AuthorizeRequest")?.ToString();
            var authorizeRequestDto =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayAuthorizeRequestDto>(tempData) : null;

            var originalAmount = authorizeRequestDto?.Amount;
            var shippingAmount = 0;

            var model = new AuthorizeConfirmViewModel
            {
                ShippingName = getAddressResponseDto.ShippingName,
                ShippingPostalCode = getAddressResponseDto.ShippingPostalCode,
                ShippingAddress1 = getAddressResponseDto.ShippingAddress1,
                ShippingAddress2 = getAddressResponseDto.ShippingAddress2,
                ShippingAddress3 = getAddressResponseDto.ShippingAddress3,
                ShippingPhone = getAddressResponseDto.ShippingPhone,
                BuyerId = getAddressResponseDto.BuyerId,
                BuyerName = getAddressResponseDto.BuyerName,
                BillingAddress1 = getAddressResponseDto.BillingAddress1,
                BillingAddress2 = getAddressResponseDto.BillingAddress2,
                BillingAddress3 = getAddressResponseDto.BillingAddress3,
                BuyerEmail = getAddressResponseDto.BuyerEmail,
                PaymentPreferences = getAddressResponseDto.PaymentPreferences,
                Amount = originalAmount,
                ShippingAmount = shippingAmount.ToString(),
                OrderId = getAddressResponseDto.OrderId,
                Mstatus = getAddressResponseDto.Mstatus,
                VResultCode = getAddressResponseDto.VResultCode,
                MerrMsg = getAddressResponseDto.MerrMsg
            };

            if (_settings.Mdk.Dummy)
            {
                model.CheckoutResultReturnUrl = authorizeRequestDto?.PayConfirmScreenUrl
                                                + "?orderId=" + HttpUtility.UrlEncode(orderId)
                                                + "&orderHash=" + HttpUtility.UrlEncode(orderHash);
            }
            else
            {
                model.CheckoutSessionId = getAddressResponseDto.CheckoutSessionId;
            }

            if (_settings.Mdk.Dummy)
            {
                return View("AuthorizeConfirmDummy", model);
            }
            else
            {
                return View("AuthorizeConfirm", model);    
            }
            
        }
        else
        {
            var model = new AuthorizeConfirmViewModel
            {
                OrderId = getAddressResponseDto.OrderId,
                Mstatus = getAddressResponseDto.Mstatus,
                VResultCode = getAddressResponseDto.VResultCode,
                MerrMsg = getAddressResponseDto.MerrMsg
            };
            return View(model);
        }
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AuthorizeConfirm(AuthorizeConfirmViewModel model)
    {
        var tempData1 = TempData.Peek(model.OrderId + "_Request")?.ToString();
        var authorizeRequestDto =
            tempData1 != null ? JsonConvert.DeserializeObject<AmazonpayAuthorizeRequestDto>(tempData1) : null;

        var tempData2 = TempData.Peek(model.OrderId + "_GetAddressResponse")?.ToString();
        var getAddressResponseDto =
            tempData2 != null ? JsonConvert.DeserializeObject<AmazonpayGetAddressResponseDto>(tempData2) : null;
        
        if (authorizeRequestDto?.Amount != model.Amount)
        {
            var requestDto = new AmazonpayUpdateAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseDto =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayUpdateAuthorizeResponseDto>(tran, requestDto);

            if (responseDto.Mstatus == "success")
            {
                return Redirect(responseDto.PayRedirectUrl);
            }
            else
            {
                TempData[model.OrderId] = JsonConvert.SerializeObject(responseDto);
                return RedirectToAction("AuthorizeConfirmResult", new {orderId = model.OrderId});
            }
        }
        else
        {
            return Redirect(getAddressResponseDto?.PayRedirectUrl);
        }

        
    }
    
    [HttpGet]
    public IActionResult AuthorizeConfirmResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayUpdateAuthorizeResponseDto>(tempData) : null;

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }
    
    #endregion

    #region Capture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AmazonpayCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayCaptureResponseDto>(tempData) : null;

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
            model.CenterOrderId = responseData?.CenterOrderId;
            model.CenterTransactionId = responseData?.CenterTransactionId;
            model.CenterResponseDatetime = responseData?.CenterResponseDatetime;
        }

        return View(model);
    }

    #endregion

    #region Cancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AmazonpayCancelRequestDto
            {
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayCancelResponseDto>(tempData) : null;

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
            model.CenterOrderId = responseData?.CenterOrderId;
            model.CenterTransactionId = responseData?.CenterTransactionId;
            model.CenterResponseDatetime = responseData?.CenterResponseDatetime;
        }

        return View(model);
    }

    #endregion

    #region Refund

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AmazonpayRefundRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayRefundResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayRefundResponseDto>(tempData) : null;

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
            model.CenterOrderId = responseData?.CenterOrderId;
            model.CenterTransactionId = responseData?.CenterTransactionId;
            model.CenterResponseDatetime = responseData?.CenterResponseDatetime;
            model.RefundableAmount = responseData?.RefundableAmount;
        }

        return View(model);
    }

    #endregion

    #region GetAddress

    [HttpGet]
    public IActionResult GetAddress()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult GetAddress(GetAddressViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AmazonpayGetAddressRequestDto
            {
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AmazonpayGetAddressResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("GetAddressResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult GetAddressResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<AmazonpayGetAddressResponseDto>(tempData) : null;

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
            model.CenterResponseDatetime = responseData?.CenterResponseDatetime;
            model.ShippingName = responseData?.ShippingName;
            model.ShippingPhone = responseData?.ShippingPhone;
            model.ShippingAddress1 = responseData?.ShippingAddress1;
            model.ShippingAddress2 = responseData?.ShippingAddress2;
            model.ShippingAddress3 = responseData?.ShippingAddress3;
            model.ShippingPrefecture = responseData?.ShippingPrefecture;
            model.ShippingPostalCode = responseData?.ShippingPostalCode;
        }

        return View(model);
    }

    #endregion

}