﻿using System.Text;
using CoreMvcApplication.Models.CarrierViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class CarrierController : Controller
{
    private readonly IHttpContextAccessor _httpContextAccessor;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public CarrierController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<CarrierController> logger,
        IHttpContextAccessor httpContextAccessor)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
        _httpContextAccessor = httpContextAccessor;
    }

    [HttpGet]
    public IActionResult Menu(uint price = 10)
    {
        return View(new MenuViewModel {Price = price});
    }

    /// <summary>
    ///     エンコーディングする文字列を返す
    ///     ・terminalKindがフィーチャーフォンの場合はShift_JIS固定
    ///     ・キャリアが、docomo,sbpsの場合はShift_JIS固定
    ///     ・上記以外はUTF-8
    /// </summary>
    /// <param name="carrier"> </param>
    /// <param name="terminalKind">
    ///     @return
    /// </param>
    private string GetEncoding(string carrier, string terminalKind)
    {
        if (terminalKind.Equals(MdkServiceConstants.CARRIER_TERMINAL_FP)) return "Shift_JIS";

        if (carrier.Equals(MdkServiceConstants.CARRIER_DOCOMO) ||
            carrier.Equals(MdkServiceConstants.CARRIER_SB_KTAI))
            return "Shift_JIS";

        return "UTF-8";
    }

    #region Authorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        return View(
            new AuthorizeViewModel
            {
                Amount = price,
                OrderId = Utils.GenerateOrderId(),
                SuccessUrl = _settings.Carrier.SuccessUrl,
                CancelUrl = _settings.Carrier.CancelUrl,
                ErrorUrl = _settings.Carrier.ErrorUrl,
                PushUrl = _settings.Mdk.Dummy ? _settings.Carrier.PushUrl : ""
            }
        );
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (model.AccountingType == MdkServiceConstants.CARRIER_MONTHLY ||
            model.AccountingType == MdkServiceConstants.CARRIER_ONDEMAND)
            // 継続課金と随時決済の場合は与信同時売上フラグを指定できないためモデルのエラーを消す
            ModelState.Remove("WithCapture");

        if (model.AccountingType == MdkServiceConstants.CARRIER_ONDEMAND)
            // 随時決済の場合は決済金額を指定できないためモデルのエラーを消す
            ModelState.Remove("Amount");

        if (model.AccountingType == MdkServiceConstants.CARRIER_ONETIME ||
            model.AccountingType == MdkServiceConstants.CARRIER_ONDEMAND)
        {
            // 都度決済と随時決済の場合は初回課金年月日と継続課金日を指定できないためモデルのエラーを消す
            ModelState.Remove("MpFirstDate");
            ModelState.Remove("MpDay");
        }

        if (model.ServiceOptionType != "sb_ktai") ModelState.Remove("D3Flag");

        if (new[] {"sb_matomete"}.Contains(model.ServiceOptionType))
        {
            ModelState.Remove("MpFirstDate");
            ModelState.Remove("MpDay");
        }


        if (!new[] {"docomo", "au"}.Contains(model.ServiceOptionType)) ModelState.Remove("OpenId");


        if (ModelState.IsValid)
        {
            var requestData = new CarrierAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString(),
                TerminalKind = model.TerminalKind,
                ItemType = model.ItemType,
                AccountingType = model.AccountingType
            };
            if (model.AccountingType == MdkServiceConstants.CARRIER_MONTHLY)
            {
                requestData.MpFirstDate = model.MpFirstDate;
                requestData.MpDay = model.MpDay;
            }

            if (model.WithCapture == MdkServiceConstants.CARRIER_AUTH_WITH_CAPTURE) requestData.WithCapture = "true";

            if (model.ServiceOptionType == MdkServiceConstants.CARRIER_SB_KTAI) requestData.D3Flag = model.D3Flag;

            requestData.ItemId = model.ItemId;
            requestData.ItemInfo = model.ItemInfo;
            requestData.SuccessUrl = model.SuccessUrl;
            requestData.CancelUrl = model.CancelUrl;
            requestData.ErrorUrl = model.ErrorUrl;
            requestData.PushUrl = model.PushUrl;
            requestData.OpenId = model.OpenId;

            if (model.ServiceOptionType == MdkServiceConstants.CARRIER_FLETS) requestData.FletsArea = model.FletsArea;

            if (model.ServiceOptionType == MdkServiceConstants.CARRIER_AU)
            {
                requestData.LoginAuId = model.LoginAuId;
            }

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierAuthorizeResponseDto>(tran, requestData);

            var responseContents = responseData.ResponseContents;
            var redirectUrl = responseData.RedirectUrl;

            if (!string.IsNullOrEmpty(responseContents))
            {
                // レスポンスコンテンツがある場合はそのまま表示します。
                // キャリアによって文字コードが異なるため、ここで文字コードを指定してください。
                var charset = GetEncoding(requestData.ServiceOptionType, requestData.TerminalKind);
                Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
                return Content(responseContents, "text/html", Encoding.GetEncoding(charset));
            }

            if (!string.IsNullOrEmpty(redirectUrl)) return Redirect(redirectUrl);

            return View("AuthorizeResult", new AuthorizeResultViewModel
            {
                MerrMsg = responseData.MerrMsg,
                Mstatus = responseData.Mstatus,
                OrderId = responseData.OrderId,
                VResultCode = responseData.VResultCode
            });
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Result([FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var queryCollection = _httpContextAccessor?.HttpContext?.Request.Query;
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(queryCollection, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            VResultCode = vResultCode
        };
        return View(model);
    }

    #endregion

    #region Terminate

    [HttpGet]
    public IActionResult Terminate()
    {
        return View(
            new TerminateViewModel
            {
                SuccessUrl = _settings.Carrier.SuccessUrl,
                CancelUrl = _settings.Carrier.CancelUrl,
                ErrorUrl = _settings.Carrier.ErrorUrl,
                PushUrl = _settings.Mdk.Dummy ? _settings.Carrier.PushUrl : ""
            }
        );
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Terminate(TerminateViewModel model)
    {
        if (model.ServiceOptionType == "sb_ktai") ModelState.Remove("TerminalKind");

        if (ModelState.IsValid)
        {
            var requestData = new CarrierTerminateRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                TerminalKind = new[] {"sb_matomete", "flets", "docomo"}.Contains(model.ServiceOptionType)
                    ? model.TerminalKind
                    : null,
                SuccessUrl = model.SuccessUrl,
                CancelUrl = model.CancelUrl,
                ErrorUrl = model.ErrorUrl,
                PushUrl = model.PushUrl
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierTerminateResponseDto>(tran, requestData);
            var responseContents = responseData.ResponseContents;

            if (!string.IsNullOrEmpty(responseContents))
            {
                // レスポンスコンテンツがある場合はそのまま表示します。
                // キャリアによって文字コードが異なるため、ここで文字コードを指定してください。
                var charset = GetEncoding(requestData.ServiceOptionType, requestData.TerminalKind);
                Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
                return Content(responseContents, "text/html", Encoding.GetEncoding(charset));
            }

            return View("TerminateResult", new TerminateResultViewModel
            {
                MerrMsg = responseData.MerrMsg,
                Mstatus = responseData.Mstatus,
                OrderId = responseData.OrderId,
                VResultCode = responseData.VResultCode,
                TerminateDatetime = responseData.TerminateDatetime
            });
        }


        return View(model);
    }

    #endregion Terminate

    #region Capture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestData = new CarrierCaptureRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                CaptureMonth = model.CaptureMonth
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierCaptureResponseDto>(tran, requestData);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new CaptureResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CarrierCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CaptureDatetime = responseData.CaptureDatetime;
        }

        return View(model);
    }

    #endregion

    #region Cancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestData = new CarrierCancelRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                CancelMonth = model.CancelMonth,
                Amount = model.Amount?.ToString()
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierCancelResponseDto>(tran, requestData);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new CancelResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CarrierCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CancelDatetime = responseData.CancelDatetime;
        }

        return View(model);
    }

    #endregion

    #region AdminTerminate

    [HttpGet]
    public IActionResult AdminTerminate()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AdminTerminate(AdminTerminateViewModel model)
    {
        if (model.ServiceOptionType == "sb_ktai") ModelState.Remove("TerminalKind");

        if (ModelState.IsValid)
        {
            var requestData = new CarrierTerminateRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId
            };

            if (new[] {"docomo", "sb_matomete"}.Contains(model.ServiceOptionType)) requestData.Force = "true";

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierTerminateResponseDto>(tran, requestData);

            return View("AdminTerminateResult", new TerminateResultViewModel
            {
                MerrMsg = responseData.MerrMsg,
                Mstatus = responseData.Mstatus,
                OrderId = responseData.OrderId,
                VResultCode = responseData.VResultCode,
                TerminateDatetime = responseData.TerminateDatetime
            });
        }


        return View(model);
    }

    #endregion AdminTerminate

    #region ReAuthorize

    [HttpGet]
    public IActionResult ReAuthorize(uint price = 10)
    {
        return View(
            new ReAuthorizeViewModel
            {
                Amount = price,
                OrderId = Utils.GenerateOrderId()
            }
        );
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult ReAuthorize(ReAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestData = new CarrierReAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                OriginalOrderId = model.OriginalOrderId,
                Amount = model.Amount?.ToString(),
                WithCapture = model.WithCapture
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CarrierReAuthorizeResponseDto>(tran, requestData);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ReAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ReAuthorizeResult(string orderId)
    {
        var model = new ReAuthorizeResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CarrierReAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.TxnTime = responseData.TxnTime;
        }

        return View(model);
    }

    #endregion
    
}