﻿using CoreMvcApplication.Models.EmViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class EmController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public EmController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<EmController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region EmMenu

    [HttpGet]
    public IActionResult Menu(uint price = 10)
    {
        var model = new MenuViewModel
        {
            Price = price
        };

        return View(model);
    }

    #endregion

    #region EmAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new EmAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            if (!new[] {"edy-pc", "edy-direct", "tcc-redirect"}.Contains(model.ServiceOptionType))
                requestDto.SettlementLimit = model.SettlementLimit;

            if (model.ServiceOptionType.StartsWith("waon")) requestDto.CancelLimit = model.CancelLimit;

            if (!new[] {"edy-pc", "edy-direct", "suica-mobile-app", "suica-pc-app", "tcc-redirect"}
                    .Contains(model.ServiceOptionType))
                requestDto.MailAddr = model.MailAddress;

            if ("waon-pc".Equals(model.ServiceOptionType))
            {
                requestDto.SuccessUrl = _settings.Em.WaonMerchantUrl + "?status=success";
                requestDto.FailureUrl = _settings.Em.WaonMerchantUrl + "?status=failure";
                requestDto.CancelUrl = _settings.Em.WaonMerchantUrl + "?status=cancel";
            }

            if (model.ServiceOptionType.StartsWith("suica")) requestDto.ScreenTitle = "SampleTitle";

            if ("tcc-redirect".Equals(model.ServiceOptionType))
            {
                requestDto.TransactionKind = "02"; // 02:電子マネー固定
                requestDto.UserId = model.UserId;
                requestDto.CompleteNoticeUrl = _settings.Em.EmCompleteNoticeUrl;
            }

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<EmAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult",
                new {orderId = model.OrderId, serviceOptionType = model.ServiceOptionType});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId, string serviceOptionType)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<EmAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ReceiptNo = responseData.ReceiptNo;
            model.AppUrl = responseData.AppUrl;
            if ("tcc-redirect".Equals(serviceOptionType)) Response.Redirect(model.AppUrl, true);

            model.LogId = responseData.LogId;
            model.CardBalance = responseData.CardBalance;
            model.CenterRequestDatetime = responseData.CenterRequestDatetime;
            model.CenterResponseDatetime = responseData.CenterResponseDatetime;
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Complete(
        string orderId, string mstatus, string vResultCode, string merrMsg,
        string completeDate, string balance, string cardNo, string cardBrandCode, string cardKind)
    {
        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            VResultCode = vResultCode,
            MerrMsg = merrMsg,
            CompleteDate = completeDate,
            Balance = balance,
            CardNo = cardNo,
            CardBrandCode = cardBrandCode,
            CardKind = cardKind
        };

        return View(model);
    }

    #endregion

    #region EmCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new EmCancelRequestDto
            {
                OrderId = model.OrderId,
                ServiceOptionType = model.ServiceOptionType,
                OrderKind = model.OrderKind
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<EmCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<EmCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.LogId = responseData.LogId;
            model.CardBalance = responseData.CardBalance;
            model.CenterRequestDatetime = responseData.CenterRequestDatetime;
            model.CenterResponseDatetime = responseData.CenterResponseDatetime;
        }

        return View(model);
    }

    #endregion

    #region EmRefund

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new EmRefundRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                Amount = model.Amount.ToString(),
                OrderKind = model.OrderKind,
                RefundOrderId = model.RefundOrderId
            };
            requestDto.OrderId = Utils.GenerateOrderId();
            if (!model.ServiceOptionType.StartsWith("edy")) requestDto.SettlementLimit = model.SettlementLimit;

            if ("waon-pc".Equals(model.ServiceOptionType))
            {
                requestDto.SuccessUrl = _settings.Em.WaonMerchantUrl + "?status=success";
                requestDto.FailureUrl = _settings.Em.WaonMerchantUrl + "?status=failure";
                requestDto.CancelUrl = _settings.Em.WaonMerchantUrl + "?status=cancel";
            }

            if (model.ServiceOptionType.StartsWith("suica")) requestDto.ScreenTitle = model.ScreenTitle;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<EmRefundResponseDto>(tran, requestDto);
            TempData[requestDto.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = requestDto.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<EmRefundResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ReceiptNo = responseData.ReceiptNo;
            model.AppUrl = responseData.AppUrl;
        }

        return View(model);
    }

    #endregion

    #region EmReAuthorize

    [HttpGet]
    public IActionResult ReAuthorize()
    {
        return View();
    }

    [HttpPost]
    public IActionResult ReAuthorize(ReAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new EmReAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId
            };
            requestDto.ReAuthorizeRedirectionUrl = _settings.Em.EmCompleteNoticeUrl;
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<EmReAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ReAuthorizeResult",
                new {orderId = model.OrderId, serviceOptionType = model.ServiceOptionType});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ReAuthorizeResult(string orderId, string serviceOptionType)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<EmReAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ReAuthAppUrl = responseData.ReAuthAppUrl;
        }

        if ("tcc-redirect".Equals(serviceOptionType)) Response.Redirect(model.ReAuthAppUrl, true);

        return View(model);
    }

    #endregion

    #region EmRemove

    [HttpGet]
    public IActionResult Remove()
    {
        var model = new RemoveViewModel
        {
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    public IActionResult Remove(RemoveViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new EmRemoveRequestDto
            {
                OrderId = model.OrderId,
                ServiceOptionType = model.ServiceOptionType,
                UserId = model.UserId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<EmRemoveResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RemoveResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RemoveResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<EmRemoveResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion
}