﻿using CoreMvcApplication.Models.LinepayViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class LinepayController : Controller
{
    private readonly IHttpContextAccessor _httpContextAccessor;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public LinepayController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<LinepayController> logger,
        IHttpContextAccessor httpContextAccessor)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
        _httpContextAccessor = httpContextAccessor;
    }

    #region LinePayAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10, string itemName = "テスト商品")
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.LinePay.SuccessUrl,
            CancelUrl = _settings.LinePay.CancelUrl,
            ItemName = itemName,
            ErrorUrl = _settings.LinePay.ErrorUrl,
            PushUrl = _settings.LinePay.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new LinepayAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WithCapture = model.WithCapture,
                ItemId = model.ItemId,
                ItemName = model.ItemName,
                ItemImageUrl = model.ItemImageUrl,
                CheckUseBrowser = model.CheckUseBrowser,
                PushUrl = model.PushUrl,
                OneTimeKey = model.OneTimeKey,
                PaymentConfirmType = model.PaymentConfirmType
            };
            if (model.PaymentConfirmType == "0")
            {
                requestDto.AppUrlScheme = model.AppUrlScheme;
                requestDto.UseOriginalApp = model.UseOriginalApp;
                requestDto.PackageName = model.PackageName;
                requestDto.SuccessUrl = model.SuccessUrl;
                requestDto.CancelUrl = model.CancelUrl;
                requestDto.ErrorUrl = model.ErrorUrl;
            }

            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<LinepayAuthorizeResponseDto>(tran, requestDto);
                var redirectUrl = responseData.RedirectWebUrl;

                // アプリからLINEを起動する場合には以下を使用してください。
                // ※使用方法については、開発ガイド参照
                // responseData.RedirectAppUrl;
                if (!string.IsNullOrEmpty(redirectUrl)) return Redirect(redirectUrl);

                var resultViewModel = new ResultViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                };
                return View("AuthorizeResult", resultViewModel);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Result(
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "linepayOrderId")] string linepayOrderId
    )
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var queryCollection = _httpContextAccessor?.HttpContext?.Request.Query;
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(queryCollection, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            VResultCode = vResultCode,
            LinePayOrderId = linepayOrderId
        };

        return View(model);
    }

    #endregion

    #region LinePayCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new LinepayCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<LinepayCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<LinepayCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CaptureDateTime = responseData.CaptureDatetime;
        }

        return View(model);
    }

    #endregion

    #region LinePayCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new LinepayCancelRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<LinepayCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<LinepayCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CancelDateTime = responseData.CancelDatetime;
        }

        return View(model);
    }

    #endregion
}