﻿using System.Text;
using CoreMvcApplication.Models.MpiViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class MpiController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public MpiController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<MpiController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }


    #region MpiAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            PushUrl = _settings.Mpi.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model,
        [FromHeader(Name = "Accept")] string accept, [FromHeader(Name = "User-Agent")] string userAgent)
    {
        if (ModelState.IsValid)
        {
            var jpo = "";
            if (new[] {"10", "80"}.Contains(model.Jpo1))
                jpo = model.Jpo1;
            else if ("61" == model.Jpo1 || string.IsNullOrWhiteSpace(model.Jpo2)) jpo = model.Jpo1 + "C" + model.Jpo2;


            var requestDto = new MpiAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Token = model.Token,
                WithCapture = model.WithCapture,
                ServiceOptionType = model.PaymentMode,
                RedirectionUri = _settings.Mpi.RedirectUrl,
                HttpUserAgent = userAgent,
                HttpAccept = accept,
                VerifyTimeout = model.verifyTimeout,
                DeviceChannel = model.deviceChannel,
                AccountType = model.accountType,
                //通常はトークン取得の際に送信しているのでDTOへのセットは不要
                //CardholderName = model.cardholderName,
                CardholderEmail = model.cardholderEmail,
                CardholderHomePhoneCountry = model.cardholderHomePhoneCountry,
                CardholderHomePhoneNumber = model.cardholderHomePhoneNumber,
                CardholderMobilePhoneCountry = model.cardholderMobilePhoneCountry,
                CardholderMobilePhoneNumber = model.cardholderMobilePhoneNumber,
                CardholderWorkPhoneCountry = model.cardholderWorkPhoneCountry,
                CardholderWorkPhoneNumber = model.cardholderWorkPhoneNumber,
                BillingAddressCity = model.billingAddressCity,
                BillingAddressCountry = model.billingAddressCountry,
                BillingAddressLine1 = model.billingAddressLine1,
                BillingAddressLine2 = model.billingAddressLine2,
                BillingAddressLine3 = model.billingAddressLine3,
                BillingPostalCode = model.billingPostalCode,
                BillingAddressState = model.billingAddressState,
                ShippingAddressCity = model.shippingAddressCity,
                ShippingAddressCountry = model.shippingAddressCountry,
                ShippingAddressLine1 = model.shippingAddressLine1,
                ShippingAddressLine2 = model.shippingAddressLine2,
                ShippingAddressLine3 = model.shippingAddressLine3,
                ShippingPostalCode = model.shippingPostalCode,
                ShippingAddressState = model.shippingAddressState,
                CustomerIp = model.customerIp,
                WithChallenge = model.withChallenge,
            };
            if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;
            if (!string.IsNullOrWhiteSpace(model.PushUrl)) requestDto.PushUrl = model.PushUrl;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MpiAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            TempData[model.OrderId + "_PaymentMode"] = model.PaymentMode;

            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MpiAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            if (responseData.Mstatus != SampleConstants.TxnFailureCode)
                return Redirect(responseData.AuthStartUrl);

            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    [HttpPost]
    public IActionResult Result([FromForm] IFormCollection posted)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(posted, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = posted["OrderId"],
            RequestId = posted["RequestId"],
            MpiMStatus = posted["mpiMstatus"],
            CardMStatus = posted["cardMstatus"],
            VResultCode = posted["vResultCode"],
            ReqCurrencyUnit = posted["reqCurrencyUnit"],
            CardTransactionType = posted["cardTransactionType"],
            CenterRequestDate = posted["centerRequestDate"],
            CenterResponseDate = posted["centerResponseDate"],
            ConnectedCenterId = posted["connectedCenterId"],
            ReqCardNumber = posted["reqCardNumber"],
            ReqAmount = posted["reqAmount"],
            AcquirerCode = posted["acquirerCode"],
            AuthCode = posted["authCode"],
            DddMessageVersion = posted["dddMessageVersion"],
            DddTransactionId = posted["dddTransactionId"],
            DddTransactionStatus = posted["dddTransactionStatus"],
            DddCavvAlgorithm = posted["dddCavvAlgorithm"],
            DddCavv = posted["dddCavv"],
            DddEci = posted["dddEci"]
        };

        return View(model);
    }


    [HttpPost]
    public IActionResult Search([FromForm(Name = "OrderId")] string orderId,
        [FromForm(Name = "RequestId")] string requestId)
    {
        // 検索用リクエストパラメータを設定
        var requestDto = new SearchRequestDto
        {
            RequestId = requestId,
            NewerFlag = SampleConstants.MdkParamFalse
        };

        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
        var responseData =
            _mdkTransactionService.ExecuteMdkTransaction<SearchResponseDto>(tran, requestDto);

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = responseData.Mstatus,
            VResultCode = responseData.VResultCode,
            MerrMsg = responseData.MerrMsg
        };

        // MPI認証結果とカード決済結果を取得します
        var transactionInfo = responseData.OrderInfos?.OrderInfo?.First()?.TransactionInfos?.TransactionInfo;

        var mpiStatus = "";
        var mpiVResultCode = "";
        var mpiTxnId = "";
        var cardStatus = "";
        var cardVResultCode = "";
        var cardTxnId = "";
        var paramPaymentMode = TempData.Peek(orderId + "_PaymentMode")?.ToString();
        if (transactionInfo != null)
            foreach (var info in transactionInfo)
            {
                // 検索結果用項目
                var mStatus = info.Mstatus;
                var vResultCode = info.VResultCode;
                var txnId = info.TxnId;
                var properInfo = info.ProperTransactionInfo;
                // 取引種別の取得
                var txnKind = "";
                if (properInfo != null) txnKind = properInfo.TxnKind;

                // MPI～カード決済までが実行された場合
                // このタイミングでリクエストIDによる検索を実行すると
                // txnKindが"mpi"と"card"のレコードがそれぞれ１件取得できる

                // MPI認証結果を取得(複数件の場合最新をチェックする)
                if (!string.IsNullOrWhiteSpace(txnId) && txnKind == "mpi")
                    if (string.IsNullOrWhiteSpace(mpiTxnId) || Convert.ToInt64(txnId) > Convert.ToInt64(mpiTxnId))
                    {
                        mpiStatus = mStatus;
                        mpiVResultCode = vResultCode;
                        mpiTxnId = txnId;
                    }

                // カード決済結果を取得（複数件の場合最新をチェックする）
                if (!string.IsNullOrWhiteSpace(txnId) && txnKind == "card")
                    if (string.IsNullOrWhiteSpace(cardTxnId) || Convert.ToInt64(txnId) > Convert.ToInt64(cardTxnId))
                    {
                        cardStatus = mStatus;
                        cardVResultCode = vResultCode;
                        cardTxnId = txnId;
                    }
            }

        // 基本は本人認証の結果
        model.Mstatus = mpiStatus;
        model.VResultCode = mpiVResultCode;

        // 本人認証に成功した場合、mStatusMPI の値は "success" になります
        if (mpiStatus == SampleConstants.TxnSuccessCode)
            // 本人認証成功で、決済種別＝「3D-Secure認証のみ(決済しない)」以外の
            // 場合、カード決済まで行われているため、完了画面に表示するのはカード決済の
            // 結果コードを表示する、というケース
            if (paramPaymentMode != "mpi-none")
            {
                model.Mstatus = cardStatus;
                model.VResultCode = cardVResultCode;
            }

        return View(model);
    }

    #endregion

    #region MpiReAuthorize

    [HttpGet]
    public IActionResult ReAuthorize(uint price = 10)
    {
        var model = new ReAuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult ReAuthorize(ReAuthorizeViewModel model,
        [FromHeader(Name = "Accept")] string accept, [FromHeader(Name = "User-Agent")] string userAgent)
    {
        if (ModelState.IsValid)
        {
            var jpo = "";
            if (new[] {"10", "80"}.Contains(model.Jpo1))
                jpo = model.Jpo1;
            else if ("61" == model.Jpo1 || string.IsNullOrWhiteSpace(model.Jpo2)) jpo = model.Jpo1 + "C" + model.Jpo2;

            var requestDto = new MpiReAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                OriginalOrderId = model.OriginalOrderId,
                WithCapture = model.WithCapture,
                ServiceOptionType = model.PaymentMode,
                RedirectionUri = _settings.Mpi.RedirectUrl,
                HttpUserAgent = userAgent,
                HttpAccept = accept,
                VerifyTimeout = model.verifyTimeout,
                DeviceChannel = model.deviceChannel,
                AccountType = model.accountType,
                //通常は元取引の値を引き継ぐためDTOへのセットは不要
                CardholderName = model.cardholderName,
                CardholderEmail = model.cardholderEmail,
                CardholderHomePhoneCountry = model.cardholderHomePhoneCountry,
                CardholderHomePhoneNumber = model.cardholderHomePhoneNumber,
                CardholderMobilePhoneCountry = model.cardholderMobilePhoneCountry,
                CardholderMobilePhoneNumber = model.cardholderMobilePhoneNumber,
                CardholderWorkPhoneCountry = model.cardholderWorkPhoneCountry,
                CardholderWorkPhoneNumber = model.cardholderWorkPhoneNumber,
                BillingAddressCity = model.billingAddressCity,
                BillingAddressCountry = model.billingAddressCountry,
                BillingAddressLine1 = model.billingAddressLine1,
                BillingAddressLine2 = model.billingAddressLine2,
                BillingAddressLine3 = model.billingAddressLine3,
                BillingPostalCode = model.billingPostalCode,
                BillingAddressState = model.billingAddressState,
                ShippingAddressCity = model.shippingAddressCity,
                ShippingAddressCountry = model.shippingAddressCountry,
                ShippingAddressLine1 = model.shippingAddressLine1,
                ShippingAddressLine2 = model.shippingAddressLine2,
                ShippingAddressLine3 = model.shippingAddressLine3,
                ShippingPostalCode = model.shippingPostalCode,
                ShippingAddressState = model.shippingAddressState,
                CustomerIp = model.customerIp,
                WithChallenge = model.withChallenge,
            };
            if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;
            if (!string.IsNullOrWhiteSpace(model.SecurityCode)) requestDto.SecurityCode = model.SecurityCode;
            if (!string.IsNullOrWhiteSpace(model.PushUrl)) requestDto.PushUrl = model.PushUrl;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MpiReAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            TempData[model.OrderId + "_PaymentMode"] = model.PaymentMode;

            return RedirectToAction("ReAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ReAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MpiReAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            if (responseData.Mstatus != SampleConstants.TxnFailureCode)
                return Redirect(responseData.AuthStartUrl);

            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region MpiGetResult

    [HttpGet]
    public IActionResult GetResult()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult GetResult(GetResultViewModel model,
        [FromHeader(Name = "Accept")] string accept, [FromHeader(Name = "User-Agent")] string userAgent)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new MpiGetResultRequestDto
            {
                OrderId = model.OrderId
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MpiGetResultResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);

            return RedirectToAction("GetResultResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult GetResultResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MpiGetResultResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId  = responseData.OrderId;
            model.Mstatus  = responseData.Mstatus;
            model.VResultCode  = responseData.VResultCode;
            model.MerrMsg  = responseData.MerrMsg;
            model.RequestId  = responseData.RequestId;
            model.ReqAmount  = responseData.ReqAmount;
            model.ReqCardNumber  = responseData.ReqCardNumber;
            model.MpiMstatus  = responseData.MpiMstatus;
            model.MpiVresultCode  = responseData.MpiVresultCode;
            model.CardMstatus  = responseData.CardMstatus;
            model.AuthCode  = responseData.AuthCode;
            model.DddMessageVersion  = responseData.DddMessageVersion;
            model.DddTransactionId  = responseData.DddTransactionId;
            model.DddDsTransactionId  = responseData.DddDsTransactionId;
            model.DddServerTransactionId  = responseData.DddServerTransactionId;
            model.DddTransactionStatus  = responseData.DddTransactionStatus;
            model.DddTransactionStatusReason  = responseData.DddTransactionStatusReason;
            model.DddCavvAlgorithm  = responseData.DddCavvAlgorithm;
            model.DddCavv  = responseData.DddCavv;
            model.DddEci  = responseData.DddEci;
        }

        return View(model);
    }

    #endregion
}