using CoreMvcApplication.Models.PayPalViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class PayPalController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public PayPalController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<PayPalController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }


    #region Authorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            TempData["PaymentAction"] = model.PaymentAction;

            if (model.PaymentAction == "authorize")
            {
                var requestDto = new PaypalAuthorizeRequestDto
                {
                    OrderId = model.OrderId,
                    Amount = model.Amount.ToString(),
                    Action = "set",
                    ReturnUrl = _settings.PayPal.DefaultReturnPage,
                    CancelUrl = _settings.PayPal.DefaultCancelPage,
                    ShippingFlag = "0"
                };

                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<PaypalAuthorizeResponseDto>(tran, requestDto);

                if (responseData.Mstatus == SampleConstants.TxnSuccessCode) return Redirect(responseData.LoginUrl);

                return View("AuthorizeInit", new AuthorizeInitViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                });
            }
            else
            {
                var requestDto = new PaypalCaptureRequestDto
                {
                    OrderId = model.OrderId,
                    Amount = model.Amount.ToString(),
                    Action = "set",
                    ReturnUrl = _settings.PayPal.DefaultReturnPage,
                    CancelUrl = _settings.PayPal.DefaultCancelPage,
                    ShippingFlag = "0"
                };

                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<PaypalCaptureResponseDto>(tran, requestDto);

                if (responseData.Mstatus == SampleConstants.TxnSuccessCode) return Redirect(responseData.LoginUrl);

                return View("AuthorizeInit", new AuthorizeInitViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                });
            }
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeConfirm([FromQuery(Name = "token")] string token,
        [FromQuery(Name = "PayerId")] string payerId)
    {
        var paymentAction = TempData["PaymentAction"]?.ToString();
        if (paymentAction == "authorize")
        {
            var requestDto = new PaypalAuthorizeRequestDto
            {
                Action = "get",
                Token = token
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalAuthorizeResponseDto>(tran, requestDto);

            var model = new AuthorizeConfirmViewModel
            {
                OrderId = responseData.OrderId,
                Mstatus = responseData.Mstatus,
                VResultCode = responseData.VResultCode,
                MerrMsg = responseData.MerrMsg,
                Token = responseData.Token,
                InvoiceId = responseData.InvoiceId,
                PayerId = responseData.PayerId,
                Amount = responseData.Amount,
                ShipName = responseData.ShipName,
                PaymentAction = "authorize"
            };

            return View(model);
        }
        else
        {
            var requestDto = new PaypalCaptureRequestDto
            {
                Action = "get",
                Token = token
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalCaptureResponseDto>(tran, requestDto);

            var model = new AuthorizeConfirmViewModel
            {
                OrderId = responseData.OrderId,
                Mstatus = responseData.Mstatus,
                VResultCode = responseData.VResultCode,
                MerrMsg = responseData.MerrMsg,
                Token = responseData.Token,
                InvoiceId = responseData.InvoiceId,
                PayerId = responseData.PayerId,
                Amount = responseData.Amount,
                ShipName = responseData.ShipName,
                PaymentAction = "capture"
            };

            return View(model);
        }
    }


    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AuthorizeComplete([FromForm(Name = "token")] string token,
        [FromForm(Name = "payerId")] string payerId, [FromForm(Name = "paymentAction")] string paymentAction)
    {
        if (paymentAction == "authorize")
        {
            var requestDto = new PaypalAuthorizeRequestDto
            {
                Action = "do",
                Token = token,
                PayerId = payerId
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalAuthorizeResponseDto>(tran, requestDto);

            var model = new AuthorizeCompleteViewModel
            {
                OrderId = responseData.OrderId,
                Mstatus = responseData.Mstatus,
                VResultCode = responseData.VResultCode,
                MerrMsg = responseData.MerrMsg,
                Token = responseData.Token,
                InvoiceId = responseData.InvoiceId,
                PayerId = responseData.PayerId,
                Amount = responseData.Amount,
                ShipName = responseData.ShipName
            };

            return View(model);
        }
        else
        {
            var requestDto = new PaypalCaptureRequestDto
            {
                Action = "do",
                Token = token,
                PayerId = payerId
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalCaptureResponseDto>(tran, requestDto);

            var model = new AuthorizeCompleteViewModel
            {
                OrderId = responseData.OrderId,
                Mstatus = responseData.Mstatus,
                VResultCode = responseData.VResultCode,
                MerrMsg = responseData.MerrMsg,
                Token = responseData.Token,
                InvoiceId = responseData.InvoiceId,
                PayerId = responseData.PayerId,
                Amount = responseData.Amount,
                ShipName = responseData.ShipName
            };

            return View(model);
        }
    }

    #endregion


    #region Capture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypalCaptureRequestDto
            {
                Action = "capture",
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypalCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion


    #region Cancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypalCancelRequestDto
            {
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypalCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion


    #region Refund

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypalRefundRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypalRefundResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypalRefundResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion
}