﻿using System.Text;
using CoreMvcApplication.Models.PayPayViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class PayPayController : Controller
{
    private readonly IHttpContextAccessor _httpContextAccessor;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public PayPayController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<PayPayController> logger,
        IHttpContextAccessor httpContextAccessor)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
        _httpContextAccessor = httpContextAccessor;
    }


    [HttpGet]
    public IActionResult Menu(uint price = 10)
    {
        return View(new MenuViewModel {Price = price});
    }


    #region result

    [HttpGet]
    public IActionResult Result(
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "paypayOrderId")] string paypayOrderId,
        [FromQuery(Name = "command")] string command,
        [FromQuery(Name = "userKey")] string userKey
    )
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var queryCollection = _httpContextAccessor?.HttpContext?.Request.Query;
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(queryCollection, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            VResultCode = vResultCode,
            PayPayOrderId = paypayOrderId,
            Command = command,
            UserKey = userKey
        };

        return View(model);
    }

    #endregion


    #region PayPayOnlineAuthorize

    [HttpGet]
    public IActionResult OnlineAuthorize(uint price = 10)
    {
        var model = new OnlineAuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.PayPay.SuccessUrl,
            CancelUrl = _settings.PayPay.CancelUrl,
            ErrorUrl = _settings.PayPay.ErrorUrl,
            PushUrl = _settings.PayPay.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult OnlineAuthorize(OnlineAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypayAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                ItemName = model.ItemName,
                ItemId = model.ItemId,
                SuccessUrl = model.SuccessUrl ?? _settings.PayPay.SuccessUrl,
                CancelUrl = model.CancelUrl ?? _settings.PayPay.CancelUrl,
                ErrorUrl = model.ErrorUrl ?? _settings.PayPay.ErrorUrl,
                PushUrl = model.PushUrl ?? _settings.PayPay.PushUrl,
                ServiceOptionType = MdkServiceConstants.PAYPAY_ONLINE
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypayAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("OnlineAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult OnlineAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var tempData = TempData.Peek(orderId)?.ToString();
            var responseData =
                tempData != null ? JsonConvert.DeserializeObject<PaypayAuthorizeResponseDto>(tempData) : null;

            if (!string.IsNullOrWhiteSpace(responseData?.ResponseContents))
            {
                Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
                return Content(responseData.ResponseContents, "text/html",
                    Encoding.GetEncoding(932) // Must respond at Shift-JIS
                );
            }

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }

    #endregion


    #region PayPayRefund

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypayRefundRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString(),
                ServiceOptionType = model.ServiceOptionType
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypayRefundResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypayRefundResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.Balance = responseData.Balance;
            model.PayPayRefundedDatetime = responseData.PaypayRefundedDatetime;
        }

        return View(model);
    }

    #endregion


    #region PayPayCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypayCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString(),
                ServiceOptionType = MdkServiceConstants.PAYPAY_ONLINE,
                OrderDescription = model.OrderDescription
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypayCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypayCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.Balance = responseData.Balance;
            model.PayPayCapturedDatetime = responseData.PaypayCapturedDatetime;
        }

        return View(model);
    }

    #endregion


    #region PayPayCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new PaypayCancelRequestDto
            {
                OrderId = model.OrderId,
                ServiceOptionType = MdkServiceConstants.PAYPAY_ONLINE
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<PaypayCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<PaypayCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.PayPayCancelledDatetime = responseData.PaypayCancelledDatetime;
        }

        return View(model);
    }

    #endregion

}