﻿using System.Text;
using CoreMvcApplication.Models.RakutenViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class RakutenController : Controller
{
    private readonly IHttpContextAccessor _httpContextAccessor;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public RakutenController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<RakutenController> logger,
        IHttpContextAccessor httpContextAccessor)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
        _httpContextAccessor = httpContextAccessor;
    }

    #region RakutenAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 100, string itemName = "テスト商品")
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.Rakuten.SuccessUrl,
            ItemName = itemName,
            ErrorUrl = _settings.Rakuten.ErrorUrl,
            PushUrl = _settings.Rakuten.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RakutenAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString(),
                PayType = model.PayType,
                WithCapture = model.WithCapture,
                AccountingType = model.AccountingType,
                ItemId = model.ItemId,
                ItemName = model.ItemName,
                SuccessUrl = model.SuccessUrl,
                ErrorUrl = model.ErrorUrl,
                PushUrl = model.PushUrl
            };

            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<RakutenAuthorizeResponseDto>(tran, requestDto);

                if (responseData.Mstatus == "success")
                    return Content(responseData.ResponseContents, "text/html", Encoding.UTF8);

                var resultViewModel = new ResultViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                };
                return View("AuthorizeResult", resultViewModel);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult Result(
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "rakutenOrderId")] string rakutenOrderId,
        [FromQuery(Name = "gatewayOrderId")] string gatewayOrderId,
        [FromQuery(Name = "rakutenConsentOrderId")] string rakutenConsentOrderId,
        [FromQuery(Name = "customerId")] string customerId,
        [FromQuery(Name = "usedPoint")] string usedPoint,
        [FromQuery(Name = "cardBrand")] string cardBrand,
        [FromQuery(Name = "cardLast4")] string cardLast4,
        [FromQuery(Name = "cardInstallments")] string cardInstallments,
        [FromQuery(Name = "cardCvc")] string cardCvc,
        [FromQuery(Name = "card3ds")] string card3ds,
        [FromQuery(Name = "transactionDatetime")]
        string transactionDatetime,
        [FromQuery(Name = "cancelExpirationTime")]
        string cancelExpirationTime,
        [FromQuery(Name = "captureExpirationTime")]
        string captureExpirationTime,
        [FromQuery(Name = "updateExpirationTime")]
        string updateExpirationTime,
        [FromQuery(Name = "extendAuthExpirationTime")]
        string extendAuthExpirationTime
    )
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var queryCollection = _httpContextAccessor?.HttpContext?.Request.Query;
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(queryCollection, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            VResultCode = vResultCode,
            RakutenOrderId = rakutenOrderId,
            GatewayOrderId = gatewayOrderId,
            RakutenConsentOrderId = rakutenConsentOrderId,
            CustomerId = customerId,
            UsedPoint = usedPoint,
            CardBrand = cardBrand,
            CardLast4 = cardLast4,
            CardInstallments = cardInstallments,
            CardCvc = cardCvc,
            Card3ds = card3ds,
            TransactionDatetime = transactionDatetime,
            CancelExpirationTime = cancelExpirationTime,
            CaptureExpirationTime = captureExpirationTime,
            UpdateExpirationTime = updateExpirationTime,
            ExtendAuthExpirationTime = extendAuthExpirationTime
        };

        return View(model);
    }

    #endregion


    #region RakutenCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RakutenCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RakutenCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RakutenCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CaptureReqDatetime = responseData.CaptureReqDatetime;
            model.TransactionDatetime = responseData.TransactionDatetime;
            model.RakutenOrderId = responseData.RakutenOrderId;
            model.GatewayOrderId = responseData.GatewayOrderId;
            model.UsedPoint = responseData.UsedPoint;
        }

        return View(model);
    }

    #endregion


    #region RakutenCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RakutenCancelRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RakutenCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RakutenCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CancelReqDatetime = responseData.CancelReqDatetime;
            model.Balance = responseData.Balance;
            model.TransactionDatetime = responseData.TransactionDatetime;
            model.RakutenOrderId = responseData.RakutenOrderId;
            model.GatewayOrderId = responseData.GatewayOrderId;
            model.UsedPoint = responseData.UsedPoint;
        }

        return View(model);
    }

    #endregion


    #region RakutenUpdateAuthorize

    [HttpGet]
    public IActionResult UpdateAuthorize()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult UpdateAuthorize(UpdateAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RakutenUpdateAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString(),
                AuthExtensionFlag = model.AuthExtensionFlag
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RakutenUpdateAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("UpdateAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult UpdateAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RakutenUpdateAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.TransactionDatetime = responseData.TransactionDatetime;
            model.RakutenOrderId = responseData.RakutenOrderId;
            model.GatewayOrderId = responseData.GatewayOrderId;
            model.Amount = responseData.Amount;
            model.UsedPoint = responseData.UsedPoint;
            model.CancelExpirationTime = responseData.CancelExpirationTime;
            model.CaptureExpirationTime = responseData.CaptureExpirationTime;
            model.UpdateExpirationTime = responseData.UpdateExpirationTime;
            model.ExtendAuthExpirationTime = responseData.ExtendAuthExpirationTime;
        }

        return View(model);
    }

    #endregion

    #region RakutenReAuthorize

    [HttpGet]
    public IActionResult ReAuthorize()
    {
        var model = new ReAuthorizeViewModel
        {
            OrderId = Utils.GenerateOrderId()
        };
        
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult ReAuthorize(ReAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RakutenReAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                OriginalOrderId = model.OriginalOrderId,
                Amount = model.Amount?.ToString(),
                WithCapture = model.WithCapture,
                ItemId = model.ItemId,
                ItemName = model.ItemName
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RakutenReAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ReAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult ReAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RakutenReAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.OriginalOrderId = responseData.OriginalOrderId;
            model.RakutenOrderId = responseData.RakutenOrderId;
            model.GatewayOrderId = responseData.GatewayOrderId;
            model.RakutenConsentOrderId = responseData.RakutenConsentOrderId;
            model.CustomerId = responseData.CustomerId;
            model.UsedPoint = responseData.UsedPoint;
            model.CardBrand = responseData.CardBrand;            
            model.CardLast4 = responseData.CardLast4;
            model.CardInstallments = responseData.CardInstallments;
            model.CardCvc = responseData.CardCvc;
            model.Card3ds = responseData.Card3ds;
            model.TransactionDatetime = responseData.TransactionDatetime;
            model.CancelExpirationTime = responseData.CancelExpirationTime;
            model.CaptureExpirationTime = responseData.CaptureExpirationTime;
            model.UpdateExpirationTime = responseData.UpdateExpirationTime;
            model.ExtendAuthExpirationTime = responseData.ExtendAuthExpirationTime;
        }

        return View(model);
    }

    #endregion
}