﻿using CoreMvcApplication.Models.RecruitViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class RecruitController : Controller
{
    private readonly IHttpContextAccessor _httpContextAccessor;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public RecruitController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<RecruitController> logger,
        IHttpContextAccessor httpContextAccessor)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
        _httpContextAccessor = httpContextAccessor;
    }

    #region RecruitAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 100, string itemName = "テスト商品")
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            SuccessUrl = _settings.Recruit.SuccessUrl,
            ItemName = itemName,
            ErrorUrl = _settings.Recruit.ErrorUrl,
            PushUrl = _settings.Recruit.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RecruitAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WithCapture = model.WithCapture,
                ItemId = model.ItemId,
                ItemName = model.ItemName,
                SuccessUrl = model.SuccessUrl,
                ErrorUrl = model.ErrorUrl,
                PushUrl = model.PushUrl
            };

            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<RecruitAuthorizeResponseDto>(tran, requestDto);

                if (responseData.Mstatus == "success") return Redirect(responseData.RedirectUrl);

                var resultViewModel = new ResultViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                };
                return View("AuthorizeResult", resultViewModel);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult Result(
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "recruitOrderId")] string recruitOrderId,
        [FromQuery(Name = "usePoint")] string usePoint,
        [FromQuery(Name = "givePoint")] string givePoint
    )
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            var queryCollection = _httpContextAccessor?.HttpContext?.Request.Query;
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(queryCollection, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = mstatus,
            UsePoint = usePoint,
            GivePoint = givePoint,
            VResultCode = vResultCode,
            RecruitOrderId = recruitOrderId
        };

        return View(model);
    }

    #endregion

    #region RecruitCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RecruitCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RecruitCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RecruitCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.UsePoint = responseData.UsePoint;
            model.GivePoint = responseData.GivePoint;
            model.Balance = responseData.Balance;
            model.CaptureDateTime = responseData.CaptureDatetime;
        }

        return View(model);
    }

    #endregion

    #region RecruitCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RecruitCancelRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RecruitCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RecruitCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.UsePoint = responseData.UsePoint;
            model.GivePoint = responseData.GivePoint;
            model.Balance = responseData.Balance;
            model.CancelDateTime = responseData.CancelDatetime;
        }

        return View(model);
    }

    #endregion

    #region RecruitExtendAuth

    [HttpGet]
    public IActionResult ExtendAuth()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult ExtendAuth(ExtendAuthViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new RecruitExtendAuthRequestDto
            {
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<RecruitExtendAuthResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ExtendAuthResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ExtendAuthResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<RecruitExtendAuthResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion
}