using CoreMvcApplication.Models.SaisonViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class SaisonController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public SaisonController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<SaisonController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };
        return View(model);
    }


    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new SaisonAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                SettlementMethod = "PC", // "PC":パソコン, "SP":スマートフォン, "MB":ガラケー
                MerchantRedirectionUri = _settings.Saison.MerchantRedirectionUri
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<SaisonAuthorizeResponseDto>(tran, requestDto);

            if (responseData.Mstatus == SampleConstants.TxnSuccessCode)
            {
                TempData[model.OrderId] = JsonConvert.SerializeObject(model);
                return Redirect(responseData.ResResponseContents);
            }

            return View("AuthorizeResult", new AuthorizeResultViewModel
            {
                MerrMsg = responseData.MerrMsg,
                Mstatus = responseData.Mstatus,
                OrderId = responseData.OrderId,
                VResultCode = responseData.VResultCode
            });
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult Capture(
        [FromQuery(Name = "serviceType")] string serviceType,
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "merrMsg")] string merrMsg,
        [FromQuery(Name = "marchTxn")] string marchTxn,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "custTxn")] string custTxn,
        [FromQuery(Name = "pointBalance")] uint? pointBalance,
        [FromQuery(Name = "aqfPointBalance")] uint? aqfPointBalance,
        [FromQuery(Name = "availableValue")] uint? availableValue
    )
    {
        var authorizeModel =
            JsonConvert.DeserializeObject<AuthorizeViewModel>(TempData.Peek(orderId).ToString());


        if (mstatus == SampleConstants.TxnSuccessCode)
        {
            var model = new CaptureViewModel
            {
                OrderId = authorizeModel.OrderId,
                PointBalance = pointBalance,
                AqfPointBalance = aqfPointBalance,
                AvailableValue = availableValue,
                Amount = authorizeModel.Amount
            };
            return View(model);
        }

        var resultViewModel = new CaptureResultViewModel
        {
            OrderId = authorizeModel.OrderId,
            Mstatus = mstatus,
            VResultCode = vResultCode,
            MerrMsg = merrMsg
        };

        TempData[authorizeModel.OrderId] = JsonConvert.SerializeObject(resultViewModel);

        return RedirectToAction("CaptureResult", new {orderId = authorizeModel.OrderId});
    }


    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new SaisonCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WalletAmount = model.WalletAmount.ToString(),
                CardAmount = model.CardAmount.ToString(),
                CardOrderId = model.CardOrderId,
                Token = model.Token,
                WithCapture =
                    model.Amount != model.WalletAmount
                        ? model.WithCapture
                        : null // 合計金額と永久不滅ウォレット決済金額が等しくない場合のみ、トークンを設定
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<SaisonCaptureResponseDto>(tran, requestDto);

            if (responseData != null)
            {
                var resultViewModel = new CaptureResultViewModel
                {
                    OrderId = responseData.OrderId,
                    Mstatus = responseData.Mstatus,
                    VResultCode = responseData.VResultCode,
                    MerrMsg = responseData.MerrMsg,
                    CardOrderId = responseData.CardOrderId,
                    CardMstatus = responseData.CardMstatus,
                    CardVResultCode = responseData.CardVResultCode,
                    CardMerrMsg = responseData.CardMerrMsg,
                    ReqCardNumber = responseData.ReqCardNumber,
                    ResAuthCode = responseData.ResAuthCode
                };

                TempData[model.OrderId] = JsonConvert.SerializeObject(resultViewModel);
            }

            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        if (TempData.ContainsKey(orderId))
        {
            var model =
                JsonConvert.DeserializeObject<CaptureResultViewModel>(TempData.Peek(orderId).ToString());
            return View(model);
        }

        return View();
    }


    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new SaisonCancelRequestDto
            {
                OrderId = model.OrderId,
                CardCancelFlag = model.CardCancelFlag
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<SaisonCancelResponseDto>(tran, requestDto);

            if (responseData != null)
            {
                var resultViewModel = new CancelResultViewModel
                {
                    OrderId = responseData.OrderId,
                    Mstatus = responseData.Mstatus,
                    VResultCode = responseData.VResultCode,
                    MerrMsg = responseData.MerrMsg,
                    CardOrderId = responseData.CardOrderId,
                    CardMstatus = responseData.CardMstatus,
                    CardVResultCode = responseData.CardVResultCode,
                    CardMerrMsg = responseData.CardMerrMsg
                };

                TempData[model.OrderId] = JsonConvert.SerializeObject(resultViewModel);
            }

            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        if (TempData.ContainsKey(orderId))
        {
            var model =
                JsonConvert.DeserializeObject<CancelResultViewModel>(TempData.Peek(orderId).ToString());
            return View(model);
        }

        return View();
    }
}