﻿using System.Text;
using CoreMvcApplication.Models.UpopViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class UpopController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;

    private readonly SampleSettings _settings;

    public UpopController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<UpopController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region Authorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new UpopAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WithCapture = model.WithCapture,
                CustomerIp = HttpContext.Features.Get<IHttpConnectionFeature>()?.RemoteIpAddress?.ToString(),
                TermUrl = _settings.Upop.TermUrl
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<UpopAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeExec", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeExec(string orderId)
    {
        var model = new ExecViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<UpopAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.EntryForm = responseData.EntryForm;
        }

        return View(model);
    }


    [HttpPost]
    public IActionResult AuthorizeResult(
        [FromForm(Name = "serviceType")] string serviceType,
        [FromForm(Name = "authInfo")] string authInfo,
        [FromForm(Name = "mstatus")] string mstatus,
        [FromForm(Name = "vResultCode")] string vResultCode,
        [FromForm(Name = "merrMsg")] string merrMsg,
        [FromForm(Name = "custTxn")] string custTxn,
        [FromForm(Name = "orderId")] string orderId,
        [FromForm(Name = "txnDatetimeJp")] string txnDatetimeJp,
        [FromForm(Name = "txnDatetimeCn")] string txnDatetimeCn,
        [FromForm(Name = "capturedAmount")] string capturedAmount,
        [FromForm(Name = "settleAmount")] string settleAmount,
        [FromForm(Name = "settleDate")] string settleDate,
        [FromForm(Name = "settleCurrency")] string settleCurrency,
        [FromForm(Name = "settleRate")] string settleRate,
        [FromForm(Name = "upopOrderId")] string upopOrderId
    )
    {
        var model = new ResultViewModel();

        var merchantSecret = _settings.Mdk.MerchantSecret;


        if (!string.IsNullOrEmpty(authInfo))
        {
            var keys = authInfo.Split('-');
            if (keys.Length == 3)
            {
                var merchantCcid = Encoding.ASCII.GetString(Util.FromBase64(keys[0]));
                var now = Encoding.ASCII.GetString(Util.FromBase64(keys[1]));
                var receivedHash = Encoding.ASCII.GetString(Util.FromBase64(keys[2]));

                var hashedData = HashUtils.GetInstance.GetDigest(new[] {merchantCcid, now, merchantSecret});

                if (hashedData == receivedHash)
                {
                    model.OrderId = orderId;
                    model.Mstatus = mstatus;
                    model.VResultCode = vResultCode;
                    model.MerrMsg = merrMsg;
                    model.CustTxn = custTxn;
                    model.TxnDatetimeJp = txnDatetimeJp;
                    model.TxnDatetimeCn = txnDatetimeCn;
                    model.CapturedAmount = capturedAmount;
                    model.SettleAmount = settleAmount;
                    model.SettleDate = settleDate;
                    model.SettleCurrency = settleCurrency;
                    model.SettleRate = settleRate;
                    model.UpopOrderId = upopOrderId;
                }
                else
                {
                    ModelState.AddModelError("AuthInfoError", "認証に失敗しました。改竄された恐れがあります。");
                }
            }
            else
            {
                ModelState.AddModelError("AuthInfoFormatError", "認証情報パラメータ(authInfo)のフォーマットが間違っています。");
            }
        }
        else
        {
            ModelState.AddModelError("AuthInfoMissing", "認証情報パラメータ(authInfo)がありません。");
        }

        return View(model);
    }

    #endregion


    #region Capture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new UpopCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<UpopCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<UpopCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CustTxn = responseData.CustTxn;
            model.TxnDatetimeJp = responseData.TxnDatetimeJp;
            model.TxnDatetimeCn = responseData.TxnDatetimeCn;
            model.CapturedAmount = responseData.CapturedAmount;
            model.SettleAmount = responseData.SettleAmount;
            model.SettleDate = responseData.SettleDate;
            model.SettleCurrency = responseData.SettleCurrency;
            model.SettleRate = responseData.SettleRate;
            model.UpopOrderId = responseData.UpopOrderId;
        }

        return View("Result", model);
    }

    #endregion


    #region Cancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new UpopCancelRequestDto
            {
                OrderId = model.OrderId
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<UpopCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<UpopCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CustTxn = responseData.CustTxn;
            model.TxnDatetimeJp = responseData.TxnDatetimeJp;
            model.TxnDatetimeCn = responseData.TxnDatetimeCn;
            model.CapturedAmount = responseData.CapturedAmount;
            model.SettleAmount = responseData.SettleAmount;
            model.SettleDate = responseData.SettleDate;
            model.SettleCurrency = responseData.SettleCurrency;
            model.SettleRate = responseData.SettleRate;
            model.UpopOrderId = responseData.UpopOrderId;
        }

        return View("Result", model);
    }

    #endregion


    #region Refund

    [HttpGet]
    public IActionResult Refund()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Refund(RefundViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new UpopRefundRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<UpopRefundResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("RefundResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RefundResult(string orderId)
    {
        var model = new ResultViewModel();
        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<UpopRefundResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CustTxn = responseData.CustTxn;
            model.TxnDatetimeJp = responseData.TxnDatetimeJp;
            model.TxnDatetimeCn = responseData.TxnDatetimeCn;
            model.CapturedAmount = responseData.CapturedAmount;
            model.SettleAmount = responseData.SettleAmount;
            model.SettleDate = responseData.SettleDate;
            model.SettleCurrency = responseData.SettleCurrency;
            model.SettleRate = responseData.SettleRate;
            model.UpopOrderId = responseData.UpopOrderId;
        }

        return View("Result", model);
    }

    #endregion
}